LIBRARY kPrf2WinApiWrappers
EXPORTS
_ActivateActCtx@8
_ActivateActCtx@8
_AddAtomA@4
_AddAtomA@4
_AddAtomW@4
_AddAtomW@4
_AddConsoleAliasA@12
_AddConsoleAliasA@12
_AddConsoleAliasW@12
_AddConsoleAliasW@12
_AddRefActCtx@4
_AddRefActCtx@4
_AddVectoredContinueHandler@8
_AddVectoredContinueHandler@8
_AddVectoredExceptionHandler@8
_AddVectoredExceptionHandler@8
_AllocConsole@0
_AllocConsole@0
_AllocateUserPhysicalPages@12
_AllocateUserPhysicalPages@12
_AreFileApisANSI@0
_AreFileApisANSI@0
_AssignProcessToJobObject@8
_AssignProcessToJobObject@8
_AttachConsole@4
_AttachConsole@4
_BackupRead@28
_BackupRead@28
_BackupSeek@24
_BackupSeek@24
_BackupWrite@28
_BackupWrite@28
_Beep@8
_Beep@8
_BeginUpdateResourceA@8
_BeginUpdateResourceA@8
_BeginUpdateResourceW@8
_BeginUpdateResourceW@8
_BindIoCompletionCallback@12
_BindIoCompletionCallback@12
_BuildCommDCBA@8
_BuildCommDCBA@8
_BuildCommDCBAndTimeoutsA@12
_BuildCommDCBAndTimeoutsA@12
_BuildCommDCBAndTimeoutsW@12
_BuildCommDCBAndTimeoutsW@12
_BuildCommDCBW@8
_BuildCommDCBW@8
_CallNamedPipeA@28
_CallNamedPipeA@28
_CallNamedPipeW@28
_CallNamedPipeW@28
_CancelDeviceWakeupRequest@4
_CancelDeviceWakeupRequest@4
_CancelIo@4
_CancelIo@4
_CancelTimerQueueTimer@8
_CancelTimerQueueTimer@8
_CancelWaitableTimer@4
_CancelWaitableTimer@4
_ChangeTimerQueueTimer@16
_ChangeTimerQueueTimer@16
_CheckNameLegalDOS8Dot3A@20
_CheckNameLegalDOS8Dot3A@20
_CheckNameLegalDOS8Dot3W@20
_CheckNameLegalDOS8Dot3W@20
_CheckRemoteDebuggerPresent@8
_CheckRemoteDebuggerPresent@8
_ClearCommBreak@4
_ClearCommBreak@4
_ClearCommError@12
_ClearCommError@12
_CloseHandle@4
_CloseHandle@4
_CommConfigDialogA@12
_CommConfigDialogA@12
_CommConfigDialogW@12
_CommConfigDialogW@12
_CompareFileTime@8
_CompareFileTime@8
_CompareStringA@24
_CompareStringA@24
_CompareStringW@24
_CompareStringW@24
_ConnectNamedPipe@8
_ConnectNamedPipe@8
_ContinueDebugEvent@12
_ContinueDebugEvent@12
_ConvertDefaultLocale@4
_ConvertDefaultLocale@4
_ConvertFiberToThread@0
_ConvertFiberToThread@0
_ConvertThreadToFiber@4
_ConvertThreadToFiber@4
_ConvertThreadToFiberEx@8
_ConvertThreadToFiberEx@8
_CopyFileA@12
_CopyFileA@12
_CopyFileExA@24
_CopyFileExA@24
_CopyFileExW@24
_CopyFileExW@24
_CopyFileW@12
_CopyFileW@12
_CreateActCtxA@4
_CreateActCtxA@4
_CreateActCtxW@4
_CreateActCtxW@4
_CreateConsoleScreenBuffer@20
_CreateConsoleScreenBuffer@20
_CreateDirectoryA@8
_CreateDirectoryA@8
_CreateDirectoryExA@12
_CreateDirectoryExA@12
_CreateDirectoryExW@12
_CreateDirectoryExW@12
_CreateDirectoryW@8
_CreateDirectoryW@8
_CreateEventA@16
_CreateEventA@16
_CreateEventW@16
_CreateEventW@16
_CreateFiber@12
_CreateFiber@12
_CreateFiberEx@20
_CreateFiberEx@20
_CreateFileA@28
_CreateFileA@28
_CreateFileMappingA@24
_CreateFileMappingA@24
_CreateFileMappingW@24
_CreateFileMappingW@24
_CreateFileW@28
_CreateFileW@28
_CreateHardLinkA@12
_CreateHardLinkA@12
_CreateHardLinkW@12
_CreateHardLinkW@12
_CreateIoCompletionPort@16
_CreateIoCompletionPort@16
_CreateJobObjectA@8
_CreateJobObjectA@8
_CreateJobObjectW@8
_CreateJobObjectW@8
_CreateJobSet@12
_CreateJobSet@12
_CreateMailslotA@16
_CreateMailslotA@16
_CreateMailslotW@16
_CreateMailslotW@16
_CreateMemoryResourceNotification@4
_CreateMemoryResourceNotification@4
_CreateMutexA@12
_CreateMutexA@12
_CreateMutexW@12
_CreateMutexW@12
_CreateNamedPipeA@32
_CreateNamedPipeA@32
_CreateNamedPipeW@32
_CreateNamedPipeW@32
_CreatePipe@16
_CreatePipe@16
_CreateProcessA@40
_CreateProcessA@40
_CreateProcessW@40
_CreateProcessW@40
_CreateRemoteThread@28
_CreateRemoteThread@28
_CreateSemaphoreA@16
_CreateSemaphoreA@16
_CreateSemaphoreW@16
_CreateSemaphoreW@16
_CreateTapePartition@16
_CreateTapePartition@16
_CreateThread@24
_CreateThread@24
_CreateTimerQueue@0
_CreateTimerQueue@0
_CreateTimerQueueTimer@28
_CreateTimerQueueTimer@28
_CreateToolhelp32Snapshot@8
_CreateToolhelp32Snapshot@8
_CreateWaitableTimerA@12
_CreateWaitableTimerA@12
_CreateWaitableTimerW@12
_CreateWaitableTimerW@12
_DeactivateActCtx@8
_DeactivateActCtx@8
_DebugActiveProcess@4
_DebugActiveProcess@4
_DebugActiveProcessStop@4
_DebugActiveProcessStop@4
_DebugBreak@0
_DebugBreak@0
_DebugBreakProcess@4
_DebugBreakProcess@4
_DebugSetProcessKillOnExit@4
_DebugSetProcessKillOnExit@4
_DecodePointer@4
_DecodePointer@4
_DecodeSystemPointer@4
_DecodeSystemPointer@4
_DefineDosDeviceA@12
_DefineDosDeviceA@12
_DefineDosDeviceW@12
_DefineDosDeviceW@12
_DeleteAtom@4
_DeleteAtom@4
_DeleteCriticalSection@4
_DeleteCriticalSection@4
_DeleteFiber@4
_DeleteFiber@4
_DeleteFileA@4
_DeleteFileA@4
_DeleteFileW@4
_DeleteFileW@4
_DeleteTimerQueue@4
_DeleteTimerQueue@4
_DeleteTimerQueueEx@8
_DeleteTimerQueueEx@8
_DeleteTimerQueueTimer@12
_DeleteTimerQueueTimer@12
_DeleteVolumeMountPointA@4
_DeleteVolumeMountPointA@4
_DeleteVolumeMountPointW@4
_DeleteVolumeMountPointW@4
_DeviceIoControl@32
_DeviceIoControl@32
_DisableThreadLibraryCalls@4
_DisableThreadLibraryCalls@4
_DisconnectNamedPipe@4
_DisconnectNamedPipe@4
_DnsHostnameToComputerNameA@12
_DnsHostnameToComputerNameA@12
_DnsHostnameToComputerNameW@12
_DnsHostnameToComputerNameW@12
_DosDateTimeToFileTime@12
_DosDateTimeToFileTime@12
_DuplicateHandle@28
_DuplicateHandle@28
_EncodePointer@4
_EncodePointer@4
_EncodeSystemPointer@4
_EncodeSystemPointer@4
_EndUpdateResourceA@8
_EndUpdateResourceA@8
_EndUpdateResourceW@8
_EndUpdateResourceW@8
_EnterCriticalSection@4
_EnterCriticalSection@4
_EnumCalendarInfoA@16
_EnumCalendarInfoA@16
_EnumCalendarInfoExA@16
_EnumCalendarInfoExA@16
_EnumCalendarInfoExW@16
_EnumCalendarInfoExW@16
_EnumCalendarInfoW@16
_EnumCalendarInfoW@16
_EnumDateFormatsA@12
_EnumDateFormatsA@12
_EnumDateFormatsExA@12
_EnumDateFormatsExA@12
_EnumDateFormatsExW@12
_EnumDateFormatsExW@12
_EnumDateFormatsW@12
_EnumDateFormatsW@12
_EnumLanguageGroupLocalesA@16
_EnumLanguageGroupLocalesA@16
_EnumLanguageGroupLocalesW@16
_EnumLanguageGroupLocalesW@16
_EnumResourceLanguagesA@20
_EnumResourceLanguagesA@20
_EnumResourceLanguagesW@20
_EnumResourceLanguagesW@20
_EnumResourceNamesA@16
_EnumResourceNamesA@16
_EnumResourceNamesW@16
_EnumResourceNamesW@16
_EnumResourceTypesA@12
_EnumResourceTypesA@12
_EnumResourceTypesW@12
_EnumResourceTypesW@12
_EnumSystemCodePagesA@8
_EnumSystemCodePagesA@8
_EnumSystemCodePagesW@8
_EnumSystemCodePagesW@8
_EnumSystemFirmwareTables@12
_EnumSystemFirmwareTables@12
_EnumSystemGeoID@12
_EnumSystemGeoID@12
_EnumSystemLanguageGroupsA@12
_EnumSystemLanguageGroupsA@12
_EnumSystemLanguageGroupsW@12
_EnumSystemLanguageGroupsW@12
_EnumSystemLocalesA@8
_EnumSystemLocalesA@8
_EnumSystemLocalesW@8
_EnumSystemLocalesW@8
_EnumTimeFormatsA@12
_EnumTimeFormatsA@12
_EnumTimeFormatsW@12
_EnumTimeFormatsW@12
_EnumUILanguagesA@12
_EnumUILanguagesA@12
_EnumUILanguagesW@12
_EnumUILanguagesW@12
_EraseTape@12
_EraseTape@12
_EscapeCommFunction@8
_EscapeCommFunction@8
_ExitProcess@4
_ExitProcess@4
_ExitThread@4
_ExitThread@4
_ExpandEnvironmentStringsA@12
_ExpandEnvironmentStringsA@12
_ExpandEnvironmentStringsW@12
_ExpandEnvironmentStringsW@12
_FatalAppExitA@8
_FatalAppExitA@8
_FatalAppExitW@8
_FatalAppExitW@8
_FatalExit@4
_FatalExit@4
_FileTimeToDosDateTime@12
_FileTimeToDosDateTime@12
_FileTimeToLocalFileTime@8
_FileTimeToLocalFileTime@8
_FileTimeToSystemTime@8
_FileTimeToSystemTime@8
_FillConsoleOutputAttribute@20
_FillConsoleOutputAttribute@20
_FillConsoleOutputCharacterA@20
_FillConsoleOutputCharacterA@20
_FillConsoleOutputCharacterW@20
_FillConsoleOutputCharacterW@20
_FindActCtxSectionGuid@20
_FindActCtxSectionGuid@20
_FindActCtxSectionStringA@20
_FindActCtxSectionStringA@20
_FindActCtxSectionStringW@20
_FindActCtxSectionStringW@20
_FindAtomA@4
_FindAtomA@4
_FindAtomW@4
_FindAtomW@4
_FindClose@4
_FindClose@4
_FindCloseChangeNotification@4
_FindCloseChangeNotification@4
_FindFirstChangeNotificationA@12
_FindFirstChangeNotificationA@12
_FindFirstChangeNotificationW@12
_FindFirstChangeNotificationW@12
_FindFirstFileA@8
_FindFirstFileA@8
_FindFirstFileExA@24
_FindFirstFileExA@24
_FindFirstFileExW@24
_FindFirstFileExW@24
_FindFirstFileW@8
_FindFirstFileW@8
_FindFirstStreamW@16
_FindFirstStreamW@16
_FindFirstVolumeA@8
_FindFirstVolumeA@8
_FindFirstVolumeMountPointA@12
_FindFirstVolumeMountPointA@12
_FindFirstVolumeMountPointW@12
_FindFirstVolumeMountPointW@12
_FindFirstVolumeW@8
_FindFirstVolumeW@8
_FindNextChangeNotification@4
_FindNextChangeNotification@4
_FindNextFileA@8
_FindNextFileA@8
_FindNextFileW@8
_FindNextFileW@8
_FindNextStreamW@8
_FindNextStreamW@8
_FindNextVolumeA@12
_FindNextVolumeA@12
_FindNextVolumeMountPointA@12
_FindNextVolumeMountPointA@12
_FindNextVolumeMountPointW@12
_FindNextVolumeMountPointW@12
_FindNextVolumeW@12
_FindNextVolumeW@12
_FindResourceA@12
_FindResourceA@12
_FindResourceExA@16
_FindResourceExA@16
_FindResourceExW@16
_FindResourceExW@16
_FindResourceW@12
_FindResourceW@12
_FindVolumeClose@4
_FindVolumeClose@4
_FindVolumeMountPointClose@4
_FindVolumeMountPointClose@4
_FlsAlloc@4
_FlsAlloc@4
_FlsFree@4
_FlsFree@4
_FlsGetValue@4
_FlsGetValue@4
_FlsSetValue@8
_FlsSetValue@8
_FlushConsoleInputBuffer@4
_FlushConsoleInputBuffer@4
_FlushFileBuffers@4
_FlushFileBuffers@4
_FlushInstructionCache@12
_FlushInstructionCache@12
_FlushViewOfFile@8
_FlushViewOfFile@8
_FoldStringA@20
_FoldStringA@20
_FoldStringW@20
_FoldStringW@20
_FormatMessageA@28
_FormatMessageA@28
_FormatMessageW@28
_FormatMessageW@28
_FreeConsole@0
_FreeConsole@0
_FreeEnvironmentStringsA@4
_FreeEnvironmentStringsA@4
_FreeEnvironmentStringsW@4
_FreeEnvironmentStringsW@4
_FreeLibrary@4
_FreeLibrary@4
_FreeLibraryAndExitThread@8
_FreeLibraryAndExitThread@8
_FreeResource@4
_FreeResource@4
_FreeUserPhysicalPages@12
_FreeUserPhysicalPages@12
_GenerateConsoleCtrlEvent@8
_GenerateConsoleCtrlEvent@8
_GetACP@0
_GetACP@0
_GetAtomNameA@12
_GetAtomNameA@12
_GetAtomNameW@12
_GetAtomNameW@12
_GetBinaryType@8
_GetBinaryType@8
_GetBinaryTypeA@8
_GetBinaryTypeA@8
_GetBinaryTypeW@8
_GetBinaryTypeW@8
_GetCPInfo@8
_GetCPInfo@8
_GetCPInfoExA@12
_GetCPInfoExA@12
_GetCPInfoExW@12
_GetCPInfoExW@12
_GetCalendarInfoA@24
_GetCalendarInfoA@24
_GetCalendarInfoW@24
_GetCalendarInfoW@24
_GetCommConfig@12
_GetCommConfig@12
_GetCommMask@8
_GetCommMask@8
_GetCommModemStatus@8
_GetCommModemStatus@8
_GetCommProperties@8
_GetCommProperties@8
_GetCommState@8
_GetCommState@8
_GetCommTimeouts@8
_GetCommTimeouts@8
_GetCommandLineA@0
_GetCommandLineA@0
_GetCommandLineW@0
_GetCommandLineW@0
_GetCompressedFileSizeA@8
_GetCompressedFileSizeA@8
_GetCompressedFileSizeW@8
_GetCompressedFileSizeW@8
_GetComputerNameA@8
_GetComputerNameA@8
_GetComputerNameExA@12
_GetComputerNameExA@12
_GetComputerNameExW@12
_GetComputerNameExW@12
_GetComputerNameW@8
_GetComputerNameW@8
_GetConsoleAliasA@16
_GetConsoleAliasA@16
_GetConsoleAliasExesA@8
_GetConsoleAliasExesA@8
_GetConsoleAliasExesLengthA@0
_GetConsoleAliasExesLengthA@0
_GetConsoleAliasExesLengthW@0
_GetConsoleAliasExesLengthW@0
_GetConsoleAliasExesW@8
_GetConsoleAliasExesW@8
_GetConsoleAliasW@16
_GetConsoleAliasW@16
_GetConsoleAliasesA@12
_GetConsoleAliasesA@12
_GetConsoleAliasesLengthA@4
_GetConsoleAliasesLengthA@4
_GetConsoleAliasesLengthW@4
_GetConsoleAliasesLengthW@4
_GetConsoleAliasesW@12
_GetConsoleAliasesW@12
_GetConsoleCP@0
_GetConsoleCP@0
_GetConsoleCursorInfo@8
_GetConsoleCursorInfo@8
_GetConsoleDisplayMode@4
_GetConsoleDisplayMode@4
_GetConsoleFontSize@8
_GetConsoleFontSize@8
_GetConsoleMode@8
_GetConsoleMode@8
_GetConsoleOutputCP@0
_GetConsoleOutputCP@0
_GetConsoleProcessList@8
_GetConsoleProcessList@8
_GetConsoleScreenBufferInfo@8
_GetConsoleScreenBufferInfo@8
_GetConsoleSelectionInfo@4
_GetConsoleSelectionInfo@4
_GetConsoleTitleA@8
_GetConsoleTitleA@8
_GetConsoleTitleW@8
_GetConsoleTitleW@8
_GetConsoleWindow@0
_GetConsoleWindow@0
_GetCurrencyFormatA@24
_GetCurrencyFormatA@24
_GetCurrencyFormatW@24
_GetCurrencyFormatW@24
_GetCurrentActCtx@4
_GetCurrentActCtx@4
_GetCurrentConsoleFont@12
_GetCurrentConsoleFont@12
_GetCurrentDirectoryA@8
_GetCurrentDirectoryA@8
_GetCurrentDirectoryW@8
_GetCurrentDirectoryW@8
_GetCurrentProcess@0
_GetCurrentProcess@0
_GetCurrentProcessId@0
_GetCurrentProcessId@0
_GetCurrentProcessorNumber@0
_GetCurrentProcessorNumber@0
_GetCurrentThread@0
_GetCurrentThread@0
_GetCurrentThreadId@0
_GetCurrentThreadId@0
_GetDateFormatA@24
_GetDateFormatA@24
_GetDateFormatW@24
_GetDateFormatW@24
_GetDefaultCommConfigA@12
_GetDefaultCommConfigA@12
_GetDefaultCommConfigW@12
_GetDefaultCommConfigW@12
_GetDevicePowerState@8
_GetDevicePowerState@8
_GetDiskFreeSpaceA@20
_GetDiskFreeSpaceA@20
_GetDiskFreeSpaceExA@16
_GetDiskFreeSpaceExA@16
_GetDiskFreeSpaceExW@16
_GetDiskFreeSpaceExW@16
_GetDiskFreeSpaceW@20
_GetDiskFreeSpaceW@20
_GetDllDirectoryA@8
_GetDllDirectoryA@8
_GetDllDirectoryW@8
_GetDllDirectoryW@8
_GetDriveTypeA@4
_GetDriveTypeA@4
_GetDriveTypeW@4
_GetDriveTypeW@4
_GetEnvironmentStrings@0
_GetEnvironmentStrings@0
_GetEnvironmentStringsA@0
_GetEnvironmentStringsA@0
_GetEnvironmentStringsW@0
_GetEnvironmentStringsW@0
_GetEnvironmentVariableA@12
_GetEnvironmentVariableA@12
_GetEnvironmentVariableW@12
_GetEnvironmentVariableW@12
_GetExitCodeProcess@8
_GetExitCodeProcess@8
_GetExitCodeThread@8
_GetExitCodeThread@8
_GetFileAttributesA@4
_GetFileAttributesA@4
_GetFileAttributesExA@12
_GetFileAttributesExA@12
_GetFileAttributesExW@12
_GetFileAttributesExW@12
_GetFileAttributesW@4
_GetFileAttributesW@4
_GetFileInformationByHandle@8
_GetFileInformationByHandle@8
_GetFileSize@8
_GetFileSize@8
_GetFileSizeEx@8
_GetFileSizeEx@8
_GetFileTime@16
_GetFileTime@16
_GetFileType@4
_GetFileType@4
_GetFirmwareEnvironmentVariableA@16
_GetFirmwareEnvironmentVariableA@16
_GetFirmwareEnvironmentVariableW@16
_GetFirmwareEnvironmentVariableW@16
_GetFullPathNameA@16
_GetFullPathNameA@16
_GetFullPathNameW@16
_GetFullPathNameW@16
_GetGeoInfoA@20
_GetGeoInfoA@20
_GetGeoInfoW@20
_GetGeoInfoW@20
_GetHandleInformation@8
_GetHandleInformation@8
_GetLargePageMinimum@0
_GetLargePageMinimum@0
_GetLargestConsoleWindowSize@4
_GetLargestConsoleWindowSize@4
_GetLastError@0
_GetLastError@0
_GetLocalTime@4
_GetLocalTime@4
_GetLocaleInfoA@16
_GetLocaleInfoA@16
_GetLocaleInfoW@16
_GetLocaleInfoW@16
_GetLogicalDriveStringsA@8
_GetLogicalDriveStringsA@8
_GetLogicalDriveStringsW@8
_GetLogicalDriveStringsW@8
_GetLogicalDrives@0
_GetLogicalDrives@0
_GetLogicalProcessorInformation@8
_GetLogicalProcessorInformation@8
_GetLongPathNameA@12
_GetLongPathNameA@12
_GetLongPathNameW@12
_GetLongPathNameW@12
_GetMailslotInfo@20
_GetMailslotInfo@20
_GetModuleFileNameA@12
_GetModuleFileNameA@12
_GetModuleFileNameW@12
_GetModuleFileNameW@12
_GetModuleHandleA@4
_GetModuleHandleA@4
_GetModuleHandleExA@12
_GetModuleHandleExA@12
_GetModuleHandleExW@12
_GetModuleHandleExW@12
_GetModuleHandleW@4
_GetModuleHandleW@4
_GetNLSVersion@12
_GetNLSVersion@12
_GetNamedPipeHandleStateA@28
_GetNamedPipeHandleStateA@28
_GetNamedPipeHandleStateW@28
_GetNamedPipeHandleStateW@28
_GetNamedPipeInfo@20
_GetNamedPipeInfo@20
_GetNativeSystemInfo@4
_GetNativeSystemInfo@4
_GetNumaAvailableMemoryNode@8
_GetNumaAvailableMemoryNode@8
_GetNumaHighestNodeNumber@4
_GetNumaHighestNodeNumber@4
_GetNumaNodeProcessorMask@8
_GetNumaNodeProcessorMask@8
_GetNumaProcessorNode@8
_GetNumaProcessorNode@8
_GetNumberFormatA@24
_GetNumberFormatA@24
_GetNumberFormatW@24
_GetNumberFormatW@24
_GetNumberOfConsoleInputEvents@8
_GetNumberOfConsoleInputEvents@8
_GetNumberOfConsoleMouseButtons@4
_GetNumberOfConsoleMouseButtons@4
_GetOEMCP@0
_GetOEMCP@0
_GetOverlappedResult@16
_GetOverlappedResult@16
_GetPriorityClass@4
_GetPriorityClass@4
_GetPrivateProfileIntA@16
_GetPrivateProfileIntA@16
_GetPrivateProfileIntW@16
_GetPrivateProfileIntW@16
_GetPrivateProfileSectionA@16
_GetPrivateProfileSectionA@16
_GetPrivateProfileSectionNamesA@12
_GetPrivateProfileSectionNamesA@12
_GetPrivateProfileSectionNamesW@12
_GetPrivateProfileSectionNamesW@12
_GetPrivateProfileSectionW@16
_GetPrivateProfileSectionW@16
_GetPrivateProfileStringA@24
_GetPrivateProfileStringA@24
_GetPrivateProfileStringW@24
_GetPrivateProfileStringW@24
_GetPrivateProfileStructA@20
_GetPrivateProfileStructA@20
_GetPrivateProfileStructW@20
_GetPrivateProfileStructW@20
_GetProcAddress@8
_GetProcAddress@8
_GetProcessAffinityMask@12
_GetProcessAffinityMask@12
_GetProcessHandleCount@8
_GetProcessHandleCount@8
_GetProcessHeap@0
_GetProcessHeap@0
_GetProcessHeaps@8
_GetProcessHeaps@8
_GetProcessId@4
_GetProcessId@4
_GetProcessIdOfThread@4
_GetProcessIdOfThread@4
_GetProcessIoCounters@8
_GetProcessIoCounters@8
_GetProcessPriorityBoost@8
_GetProcessPriorityBoost@8
_GetProcessShutdownParameters@8
_GetProcessShutdownParameters@8
_GetProcessTimes@20
_GetProcessTimes@20
_GetProcessVersion@4
_GetProcessVersion@4
_GetProcessWorkingSetSize@12
_GetProcessWorkingSetSize@12
_GetProcessWorkingSetSizeEx@16
_GetProcessWorkingSetSizeEx@16
_GetProfileIntA@12
_GetProfileIntA@12
_GetProfileIntW@12
_GetProfileIntW@12
_GetProfileSectionA@12
_GetProfileSectionA@12
_GetProfileSectionW@12
_GetProfileSectionW@12
_GetProfileStringA@20
_GetProfileStringA@20
_GetProfileStringW@20
_GetProfileStringW@20
_GetQueuedCompletionStatus@20
_GetQueuedCompletionStatus@20
_GetShortPathNameA@12
_GetShortPathNameA@12
_GetShortPathNameW@12
_GetShortPathNameW@12
_GetStartupInfoA@4
_GetStartupInfoA@4
_GetStartupInfoW@4
_GetStartupInfoW@4
_GetStdHandle@4
_GetStdHandle@4
_GetStringTypeA@20
_GetStringTypeA@20
_GetStringTypeExA@20
_GetStringTypeExA@20
_GetStringTypeExW@20
_GetStringTypeExW@20
_GetStringTypeW@16
_GetStringTypeW@16
_GetSystemDefaultLCID@0
_GetSystemDefaultLCID@0
_GetSystemDefaultLangID@0
_GetSystemDefaultLangID@0
_GetSystemDefaultUILanguage@0
_GetSystemDefaultUILanguage@0
_GetSystemDirectoryA@8
_GetSystemDirectoryA@8
_GetSystemDirectoryW@8
_GetSystemDirectoryW@8
_GetSystemFileCacheSize@12
_GetSystemFileCacheSize@12
_GetSystemFirmwareTable@16
_GetSystemFirmwareTable@16
_GetSystemInfo@4
_GetSystemInfo@4
_GetSystemPowerStatus@4
_GetSystemPowerStatus@4
_GetSystemRegistryQuota@8
_GetSystemRegistryQuota@8
_GetSystemTime@4
_GetSystemTime@4
_GetSystemTimeAdjustment@12
_GetSystemTimeAdjustment@12
_GetSystemTimeAsFileTime@4
_GetSystemTimeAsFileTime@4
_GetSystemTimes@12
_GetSystemTimes@12
_GetSystemWindowsDirectoryA@8
_GetSystemWindowsDirectoryA@8
_GetSystemWindowsDirectoryW@8
_GetSystemWindowsDirectoryW@8
_GetSystemWow64DirectoryA@8
_GetSystemWow64DirectoryA@8
_GetSystemWow64DirectoryW@8
_GetSystemWow64DirectoryW@8
_GetTapeParameters@16
_GetTapeParameters@16
_GetTapePosition@20
_GetTapePosition@20
_GetTapeStatus@4
_GetTapeStatus@4
_GetTempFileNameA@16
_GetTempFileNameA@16
_GetTempFileNameW@16
_GetTempFileNameW@16
_GetTempPathA@8
_GetTempPathA@8
_GetTempPathW@8
_GetTempPathW@8
_GetThreadContext@8
_GetThreadContext@8
_GetThreadIOPendingFlag@8
_GetThreadIOPendingFlag@8
_GetThreadId@4
_GetThreadId@4
_GetThreadLocale@0
_GetThreadLocale@0
_GetThreadPriority@4
_GetThreadPriority@4
_GetThreadPriorityBoost@8
_GetThreadPriorityBoost@8
_GetThreadSelectorEntry@12
_GetThreadSelectorEntry@12
_GetThreadTimes@20
_GetThreadTimes@20
_GetTickCount@0
_GetTickCount@0
_GetTimeFormatA@24
_GetTimeFormatA@24
_GetTimeFormatW@24
_GetTimeFormatW@24
_GetTimeZoneInformation@4
_GetTimeZoneInformation@4
_GetUserDefaultLCID@0
_GetUserDefaultLCID@0
_GetUserDefaultLangID@0
_GetUserDefaultLangID@0
_GetUserDefaultUILanguage@0
_GetUserDefaultUILanguage@0
_GetUserGeoID@4
_GetUserGeoID@4
_GetVersion@0
_GetVersion@0
_GetVersionExA@4
_GetVersionExA@4
_GetVersionExW@4
_GetVersionExW@4
_GetVolumeInformationA@32
_GetVolumeInformationA@32
_GetVolumeInformationW@32
_GetVolumeInformationW@32
_GetVolumeNameForVolumeMountPointA@12
_GetVolumeNameForVolumeMountPointA@12
_GetVolumeNameForVolumeMountPointW@12
_GetVolumeNameForVolumeMountPointW@12
_GetVolumePathNameA@12
_GetVolumePathNameA@12
_GetVolumePathNameW@12
_GetVolumePathNameW@12
_GetVolumePathNamesForVolumeNameA@16
_GetVolumePathNamesForVolumeNameA@16
_GetVolumePathNamesForVolumeNameW@16
_GetVolumePathNamesForVolumeNameW@16
_GetWindowsDirectoryA@8
_GetWindowsDirectoryA@8
_GetWindowsDirectoryW@8
_GetWindowsDirectoryW@8
_GetWriteWatch@24
_GetWriteWatch@24
_GlobalAddAtomA@4
_GlobalAddAtomA@4
_GlobalAddAtomW@4
_GlobalAddAtomW@4
_GlobalAlloc@8
_GlobalAlloc@8
_GlobalCompact@4
_GlobalCompact@4
_GlobalDeleteAtom@4
_GlobalDeleteAtom@4
_GlobalFindAtomA@4
_GlobalFindAtomA@4
_GlobalFindAtomW@4
_GlobalFindAtomW@4
_GlobalFix@4
_GlobalFix@4
_GlobalFlags@4
_GlobalFlags@4
_GlobalFree@4
_GlobalFree@4
_GlobalGetAtomNameA@12
_GlobalGetAtomNameA@12
_GlobalGetAtomNameW@12
_GlobalGetAtomNameW@12
_GlobalHandle@4
_GlobalHandle@4
_GlobalLock@4
_GlobalLock@4
_GlobalMemoryStatus@4
_GlobalMemoryStatus@4
_GlobalMemoryStatusEx@4
_GlobalMemoryStatusEx@4
_GlobalReAlloc@12
_GlobalReAlloc@12
_GlobalSize@4
_GlobalSize@4
_GlobalUnWire@4
_GlobalUnWire@4
_GlobalUnfix@4
_GlobalUnfix@4
_GlobalUnlock@4
_GlobalUnlock@4
_GlobalWire@4
_GlobalWire@4
_Heap32First@12
_Heap32First@12
_Heap32ListFirst@8
_Heap32ListFirst@8
_Heap32ListNext@8
_Heap32ListNext@8
_Heap32Next@4
_Heap32Next@4
_HeapAlloc@12
_HeapAlloc@12
_HeapCompact@8
_HeapCompact@8
_HeapCreate@12
_HeapCreate@12
_HeapDestroy@4
_HeapDestroy@4
_HeapFree@12
_HeapFree@12
_HeapLock@4
_HeapLock@4
_HeapQueryInformation@20
_HeapQueryInformation@20
_HeapReAlloc@16
_HeapReAlloc@16
_HeapSetInformation@16
_HeapSetInformation@16
_HeapSize@12
_HeapSize@12
_HeapUnlock@4
_HeapUnlock@4
_HeapValidate@12
_HeapValidate@12
_HeapWalk@8
_HeapWalk@8
_InitAtomTable@4
_InitAtomTable@4
_InitializeCriticalSection@4
_InitializeCriticalSection@4
_InitializeCriticalSectionAndSpinCount@8
_InitializeCriticalSectionAndSpinCount@8
_InitializeSListHead@4
_InitializeSListHead@4
_InterlockedCompareExchange64@20
_InterlockedCompareExchange64@20
_InterlockedCompareExchange@12
_InterlockedCompareExchange@12
_InterlockedDecrement@4
_InterlockedDecrement@4
_InterlockedExchange@8
_InterlockedExchange@8
_InterlockedExchangeAdd@8
_InterlockedExchangeAdd@8
_InterlockedFlushSList@4
_InterlockedFlushSList@4
_InterlockedIncrement@4
_InterlockedIncrement@4
_InterlockedPopEntrySList@4
_InterlockedPopEntrySList@4
_InterlockedPushEntrySList@8
_InterlockedPushEntrySList@8
_IsBadCodePtr@4
_IsBadCodePtr@4
_IsBadHugeReadPtr@8
_IsBadHugeReadPtr@8
_IsBadHugeWritePtr@8
_IsBadHugeWritePtr@8
_IsBadReadPtr@8
_IsBadReadPtr@8
_IsBadStringPtrA@8
_IsBadStringPtrA@8
_IsBadStringPtrW@8
_IsBadStringPtrW@8
_IsBadWritePtr@8
_IsBadWritePtr@8
_IsDBCSLeadByte@4
_IsDBCSLeadByte@4
_IsDBCSLeadByteEx@8
_IsDBCSLeadByteEx@8
_IsDebuggerPresent@0
_IsDebuggerPresent@0
_IsNLSDefinedString@20
_IsNLSDefinedString@20
_IsProcessInJob@12
_IsProcessInJob@12
_IsProcessorFeaturePresent@4
_IsProcessorFeaturePresent@4
_IsSystemResumeAutomatic@0
_IsSystemResumeAutomatic@0
_IsValidCodePage@4
_IsValidCodePage@4
_IsValidLanguageGroup@8
_IsValidLanguageGroup@8
_IsValidLocale@8
_IsValidLocale@8
_IsWow64Process@8
_IsWow64Process@8
_LCMapStringA@24
_LCMapStringA@24
_LCMapStringW@24
_LCMapStringW@24
_LeaveCriticalSection@4
_LeaveCriticalSection@4
_LoadLibraryA@4
_LoadLibraryA@4
_LoadLibraryExA@12
_LoadLibraryExA@12
_LoadLibraryExW@12
_LoadLibraryExW@12
_LoadLibraryW@4
_LoadLibraryW@4
_LoadModule@8
_LoadModule@8
_LoadResource@8
_LoadResource@8
_LocalAlloc@8
_LocalAlloc@8
_LocalCompact@4
_LocalCompact@4
_LocalFileTimeToFileTime@8
_LocalFileTimeToFileTime@8
_LocalFlags@4
_LocalFlags@4
_LocalFree@4
_LocalFree@4
_LocalHandle@4
_LocalHandle@4
_LocalLock@4
_LocalLock@4
_LocalReAlloc@12
_LocalReAlloc@12
_LocalShrink@8
_LocalShrink@8
_LocalSize@4
_LocalSize@4
_LocalUnlock@4
_LocalUnlock@4
_LockFile@20
_LockFile@20
_LockFileEx@24
_LockFileEx@24
_LockResource@4
_LockResource@4
_MapUserPhysicalPages@12
_MapUserPhysicalPages@12
_MapUserPhysicalPagesScatter@12
_MapUserPhysicalPagesScatter@12
_MapViewOfFile@20
_MapViewOfFile@20
_MapViewOfFileEx@24
_MapViewOfFileEx@24
_Module32First@8
_Module32First@8
_Module32FirstW@8
_Module32FirstW@8
_Module32Next@8
_Module32Next@8
_Module32NextW@8
_Module32NextW@8
_MoveFileA@8
_MoveFileA@8
_MoveFileExA@12
_MoveFileExA@12
_MoveFileExW@12
_MoveFileExW@12
_MoveFileW@8
_MoveFileW@8
_MoveFileWithProgressA@20
_MoveFileWithProgressA@20
_MoveFileWithProgressW@20
_MoveFileWithProgressW@20
_MulDiv@12
_MulDiv@12
_MultiByteToWideChar@24
_MultiByteToWideChar@24
_NeedCurrentDirectoryForExePathA@4
_NeedCurrentDirectoryForExePathA@4
_NeedCurrentDirectoryForExePathW@4
_NeedCurrentDirectoryForExePathW@4
_OpenEventA@12
_OpenEventA@12
_OpenEventW@12
_OpenEventW@12
_OpenFile@12
_OpenFile@12
_OpenFileMappingA@12
_OpenFileMappingA@12
_OpenFileMappingW@12
_OpenFileMappingW@12
_OpenJobObjectA@12
_OpenJobObjectA@12
_OpenJobObjectW@12
_OpenJobObjectW@12
_OpenMutexA@12
_OpenMutexA@12
_OpenMutexW@12
_OpenMutexW@12
_OpenProcess@12
_OpenProcess@12
_OpenSemaphoreA@12
_OpenSemaphoreA@12
_OpenSemaphoreW@12
_OpenSemaphoreW@12
_OpenThread@12
_OpenThread@12
_OpenWaitableTimerA@12
_OpenWaitableTimerA@12
_OpenWaitableTimerW@12
_OpenWaitableTimerW@12
_OutputDebugStringA@4
_OutputDebugStringA@4
_OutputDebugStringW@4
_OutputDebugStringW@4
_PeekConsoleInputA@16
_PeekConsoleInputA@16
_PeekConsoleInputW@16
_PeekConsoleInputW@16
_PeekNamedPipe@24
_PeekNamedPipe@24
_PostQueuedCompletionStatus@16
_PostQueuedCompletionStatus@16
_PrepareTape@12
_PrepareTape@12
_Process32First@8
_Process32First@8
_Process32FirstW@8
_Process32FirstW@8
_Process32Next@8
_Process32Next@8
_Process32NextW@8
_Process32NextW@8
_ProcessIdToSessionId@8
_ProcessIdToSessionId@8
_PulseEvent@4
_PulseEvent@4
_PurgeComm@8
_PurgeComm@8
_QueryActCtxW@28
_QueryActCtxW@28
_QueryDepthSList@4
_QueryDepthSList@4
_QueryDosDeviceA@12
_QueryDosDeviceA@12
_QueryDosDeviceW@12
_QueryDosDeviceW@12
_QueryInformationJobObject@20
_QueryInformationJobObject@20
_QueryMemoryResourceNotification@8
_QueryMemoryResourceNotification@8
_QueryPerformanceCounter@4
_QueryPerformanceCounter@4
_QueryPerformanceFrequency@4
_QueryPerformanceFrequency@4
_QueueUserAPC@12
_QueueUserAPC@12
_QueueUserWorkItem@12
_QueueUserWorkItem@12
_RaiseException@16
_RaiseException@16
_ReOpenFile@16
_ReOpenFile@16
_ReadConsoleA@20
_ReadConsoleA@20
_ReadConsoleInputA@16
_ReadConsoleInputA@16
_ReadConsoleInputW@16
_ReadConsoleInputW@16
_ReadConsoleOutputA@20
_ReadConsoleOutputA@20
_ReadConsoleOutputAttribute@20
_ReadConsoleOutputAttribute@20
_ReadConsoleOutputCharacterA@20
_ReadConsoleOutputCharacterA@20
_ReadConsoleOutputCharacterW@20
_ReadConsoleOutputCharacterW@20
_ReadConsoleOutputW@20
_ReadConsoleOutputW@20
_ReadConsoleW@20
_ReadConsoleW@20
_ReadDirectoryChangesW@32
_ReadDirectoryChangesW@32
_ReadFile@20
_ReadFile@20
_ReadFileEx@20
_ReadFileEx@20
_ReadFileScatter@20
_ReadFileScatter@20
_ReadProcessMemory@20
_ReadProcessMemory@20
_RegisterWaitForSingleObject@24
_RegisterWaitForSingleObject@24
_RegisterWaitForSingleObjectEx@20
_RegisterWaitForSingleObjectEx@20
_ReleaseActCtx@4
_ReleaseActCtx@4
_ReleaseMutex@4
_ReleaseMutex@4
_ReleaseSemaphore@12
_ReleaseSemaphore@12
_RemoveDirectoryA@4
_RemoveDirectoryA@4
_RemoveDirectoryW@4
_RemoveDirectoryW@4
_RemoveVectoredContinueHandler@4
_RemoveVectoredContinueHandler@4
_RemoveVectoredExceptionHandler@4
_RemoveVectoredExceptionHandler@4
_ReplaceFile@24
_ReplaceFile@24
_ReplaceFileA@24
_ReplaceFileA@24
_ReplaceFileW@24
_ReplaceFileW@24
_RequestDeviceWakeup@4
_RequestDeviceWakeup@4
_RequestWakeupLatency@4
_RequestWakeupLatency@4
_ResetEvent@4
_ResetEvent@4
_ResetWriteWatch@8
_ResetWriteWatch@8
_RestoreLastError@4
_RestoreLastError@4
_ResumeThread@4
_ResumeThread@4
_RtlCaptureContext@4
_RtlCaptureContext@4
_RtlCaptureStackBackTrace@16
_RtlCaptureStackBackTrace@16
_RtlFillMemory@12
_RtlFillMemory@12
_RtlMoveMemory@12
_RtlMoveMemory@12
_RtlUnwind@16
_RtlUnwind@16
_RtlZeroMemory@8
_RtlZeroMemory@8
_ScrollConsoleScreenBufferA@20
_ScrollConsoleScreenBufferA@20
_ScrollConsoleScreenBufferW@20
_ScrollConsoleScreenBufferW@20
_SearchPathA@24
_SearchPathA@24
_SearchPathW@24
_SearchPathW@24
_SetCalendarInfoA@16
_SetCalendarInfoA@16
_SetCalendarInfoW@16
_SetCalendarInfoW@16
_SetCommBreak@4
_SetCommBreak@4
_SetCommConfig@12
_SetCommConfig@12
_SetCommMask@8
_SetCommMask@8
_SetCommState@8
_SetCommState@8
_SetCommTimeouts@8
_SetCommTimeouts@8
_SetComputerNameA@4
_SetComputerNameA@4
_SetComputerNameExA@8
_SetComputerNameExA@8
_SetComputerNameExW@8
_SetComputerNameExW@8
_SetComputerNameW@4
_SetComputerNameW@4
_SetConsoleActiveScreenBuffer@4
_SetConsoleActiveScreenBuffer@4
_SetConsoleCP@4
_SetConsoleCP@4
_SetConsoleCtrlHandler@8
_SetConsoleCtrlHandler@8
_SetConsoleCursor@8
_SetConsoleCursor@8
_SetConsoleCursorInfo@8
_SetConsoleCursorInfo@8
_SetConsoleCursorPosition@8
_SetConsoleCursorPosition@8
_SetConsoleMode@8
_SetConsoleMode@8
_SetConsoleOutputCP@4
_SetConsoleOutputCP@4
_SetConsoleScreenBufferSize@8
_SetConsoleScreenBufferSize@8
_SetConsoleTextAttribute@8
_SetConsoleTextAttribute@8
_SetConsoleTitleA@4
_SetConsoleTitleA@4
_SetConsoleTitleW@4
_SetConsoleTitleW@4
_SetConsoleWindowInfo@12
_SetConsoleWindowInfo@12
_SetCriticalSectionSpinCount@8
_SetCriticalSectionSpinCount@8
_SetCurrentDirectoryA@4
_SetCurrentDirectoryA@4
_SetCurrentDirectoryW@4
_SetCurrentDirectoryW@4
_SetDefaultCommConfigA@12
_SetDefaultCommConfigA@12
_SetDefaultCommConfigW@12
_SetDefaultCommConfigW@12
_SetDllDirectoryA@4
_SetDllDirectoryA@4
_SetDllDirectoryW@4
_SetDllDirectoryW@4
_SetEndOfFile@4
_SetEndOfFile@4
_SetEnvironmentStringsA@4
_SetEnvironmentStringsA@4
_SetEnvironmentStringsW@4
_SetEnvironmentStringsW@4
_SetEnvironmentVariableA@8
_SetEnvironmentVariableA@8
_SetEnvironmentVariableW@8
_SetEnvironmentVariableW@8
_SetErrorMode@4
_SetErrorMode@4
_SetEvent@4
_SetEvent@4
_SetFileApisToANSI@0
_SetFileApisToANSI@0
_SetFileApisToOEM@0
_SetFileApisToOEM@0
_SetFileAttributesA@8
_SetFileAttributesA@8
_SetFileAttributesW@8
_SetFileAttributesW@8
_SetFilePointer@16
_SetFilePointer@16
_SetFilePointerEx@20
_SetFilePointerEx@20
_SetFileShortNameA@8
_SetFileShortNameA@8
_SetFileShortNameW@8
_SetFileShortNameW@8
_SetFileTime@16
_SetFileTime@16
_SetFileValidData@12
_SetFileValidData@12
_SetFirmwareEnvironmentVariableA@16
_SetFirmwareEnvironmentVariableA@16
_SetFirmwareEnvironmentVariableW@16
_SetFirmwareEnvironmentVariableW@16
_SetHandleCount@4
_SetHandleCount@4
_SetHandleInformation@12
_SetHandleInformation@12
_SetInformationJobObject@16
_SetInformationJobObject@16
_SetLastError@4
_SetLastError@4
_SetLocalTime@4
_SetLocalTime@4
_SetLocaleInfoA@12
_SetLocaleInfoA@12
_SetLocaleInfoW@12
_SetLocaleInfoW@12
_SetMailslotInfo@8
_SetMailslotInfo@8
_SetMessageWaitingIndicator@8
_SetMessageWaitingIndicator@8
_SetNamedPipeHandleState@16
_SetNamedPipeHandleState@16
_SetPriorityClass@8
_SetPriorityClass@8
_SetProcessAffinityMask@8
_SetProcessAffinityMask@8
_SetProcessPriorityBoost@8
_SetProcessPriorityBoost@8
_SetProcessShutdownParameters@8
_SetProcessShutdownParameters@8
_SetProcessWorkingSetSize@12
_SetProcessWorkingSetSize@12
_SetProcessWorkingSetSizeEx@16
_SetProcessWorkingSetSizeEx@16
_SetStdHandle@8
_SetStdHandle@8
_SetSystemFileCacheSize@12
_SetSystemFileCacheSize@12
_SetSystemPowerState@8
_SetSystemPowerState@8
_SetSystemTime@4
_SetSystemTime@4
_SetSystemTimeAdjustment@8
_SetSystemTimeAdjustment@8
_SetTapeParameters@12
_SetTapeParameters@12
_SetTapePosition@24
_SetTapePosition@24
_SetThreadAffinityMask@8
_SetThreadAffinityMask@8
_SetThreadContext@8
_SetThreadContext@8
_SetThreadExecutionState@4
_SetThreadExecutionState@4
_SetThreadIdealProcessor@8
_SetThreadIdealProcessor@8
_SetThreadLocale@4
_SetThreadLocale@4
_SetThreadPriority@8
_SetThreadPriority@8
_SetThreadPriorityBoost@8
_SetThreadPriorityBoost@8
_SetThreadStackGuarantee@4
_SetThreadStackGuarantee@4
_SetTimeZoneInformation@4
_SetTimeZoneInformation@4
_SetTimerQueueTimer@24
_SetTimerQueueTimer@24
_SetUnhandledExceptionFilter@4
_SetUnhandledExceptionFilter@4
_SetUserGeoID@4
_SetUserGeoID@4
_SetVolumeLabelA@8
_SetVolumeLabelA@8
_SetVolumeLabelW@8
_SetVolumeLabelW@8
_SetVolumeMountPointA@8
_SetVolumeMountPointA@8
_SetVolumeMountPointW@8
_SetVolumeMountPointW@8
_SetWaitableTimer@24
_SetWaitableTimer@24
_SetupComm@12
_SetupComm@12
_SignalObjectAndWait@16
_SignalObjectAndWait@16
_SizeofResource@8
_SizeofResource@8
_Sleep@4
_Sleep@4
_SleepEx@8
_SleepEx@8
_SuspendThread@4
_SuspendThread@4
_SwitchToFiber@4
_SwitchToFiber@4
_SwitchToThread@0
_SwitchToThread@0
_SystemTimeToFileTime@8
_SystemTimeToFileTime@8
_SystemTimeToTzSpecificLocalTime@12
_SystemTimeToTzSpecificLocalTime@12
_TerminateJobObject@8
_TerminateJobObject@8
_TerminateProcess@8
_TerminateProcess@8
_TerminateThread@8
_TerminateThread@8
_Thread32First@8
_Thread32First@8
_Thread32Next@8
_Thread32Next@8
_TlsAlloc@0
_TlsAlloc@0
_TlsFree@4
_TlsFree@4
_TlsGetValue@4
_TlsGetValue@4
_TlsSetValue@8
_TlsSetValue@8
_Toolhelp32ReadProcessMemory@20
_Toolhelp32ReadProcessMemory@20
_TransactNamedPipe@28
_TransactNamedPipe@28
_TransmitCommChar@8
_TransmitCommChar@8
_TryEnterCriticalSection@4
_TryEnterCriticalSection@4
_TzSpecificLocalTimeToSystemTime@12
_TzSpecificLocalTimeToSystemTime@12
_UnhandledExceptionFilter@4
_UnhandledExceptionFilter@4
_UnlockFile@20
_UnlockFile@20
_UnlockFileEx@20
_UnlockFileEx@20
_UnmapViewOfFile@4
_UnmapViewOfFile@4
_UnregisterWait@4
_UnregisterWait@4
_UnregisterWaitEx@8
_UnregisterWaitEx@8
_UpdateResourceA@24
_UpdateResourceA@24
_UpdateResourceW@24
_UpdateResourceW@24
_VerLanguageNameA@12
_VerLanguageNameA@12
_VerLanguageNameW@12
_VerLanguageNameW@12
_VerSetConditionMask@16
_VerSetConditionMask@16
_VerifyVersionInfoA@16
_VerifyVersionInfoA@16
_VerifyVersionInfoW@16
_VerifyVersionInfoW@16
_VirtualAlloc@16
_VirtualAlloc@16
_VirtualAllocEx@20
_VirtualAllocEx@20
_VirtualFree@12
_VirtualFree@12
_VirtualFreeEx@16
_VirtualFreeEx@16
_VirtualLock@8
_VirtualLock@8
_VirtualProtect@16
_VirtualProtect@16
_VirtualProtectEx@20
_VirtualProtectEx@20
_VirtualQuery@12
_VirtualQuery@12
_VirtualQueryEx@16
_VirtualQueryEx@16
_VirtualUnlock@8
_VirtualUnlock@8
_WTSGetActiveConsoleSessionId@0
_WTSGetActiveConsoleSessionId@0
_WaitCommEvent@12
_WaitCommEvent@12
_WaitForDebugEvent@8
_WaitForDebugEvent@8
_WaitForMultipleObjects@16
_WaitForMultipleObjects@16
_WaitForMultipleObjectsEx@20
_WaitForMultipleObjectsEx@20
_WaitForSingleObject@8
_WaitForSingleObject@8
_WaitForSingleObjectEx@12
_WaitForSingleObjectEx@12
_WaitNamedPipeA@8
_WaitNamedPipeA@8
_WaitNamedPipeW@8
_WaitNamedPipeW@8
_WideCharToMultiByte@32
_WideCharToMultiByte@32
_WinExec@8
_WinExec@8
_Wow64DisableWow64FsRedirection@4
_Wow64DisableWow64FsRedirection@4
_Wow64EnableWow64FsRedirection@4
_Wow64EnableWow64FsRedirection@4
_Wow64RevertWow64FsRedirection@4
_Wow64RevertWow64FsRedirection@4
_WriteConsoleA@20
_WriteConsoleA@20
_WriteConsoleInputA@16
_WriteConsoleInputA@16
_WriteConsoleInputW@16
_WriteConsoleInputW@16
_WriteConsoleOutputA@20
_WriteConsoleOutputA@20
_WriteConsoleOutputAttribute@20
_WriteConsoleOutputAttribute@20
_WriteConsoleOutputCharacterA@20
_WriteConsoleOutputCharacterA@20
_WriteConsoleOutputCharacterW@20
_WriteConsoleOutputCharacterW@20
_WriteConsoleOutputW@20
_WriteConsoleOutputW@20
_WriteConsoleW@20
_WriteConsoleW@20
_WriteFile@20
_WriteFile@20
_WriteFileEx@20
_WriteFileEx@20
_WriteFileGather@20
_WriteFileGather@20
_WritePrivateProfileSectionA@12
_WritePrivateProfileSectionA@12
_WritePrivateProfileSectionW@12
_WritePrivateProfileSectionW@12
_WritePrivateProfileStringA@16
_WritePrivateProfileStringA@16
_WritePrivateProfileStringW@16
_WritePrivateProfileStringW@16
_WritePrivateProfileStructA@20
_WritePrivateProfileStructA@20
_WritePrivateProfileStructW@20
_WritePrivateProfileStructW@20
_WriteProcessMemory@20
_WriteProcessMemory@20
_WriteProfileSectionA@8
_WriteProfileSectionA@8
_WriteProfileSectionW@8
_WriteProfileSectionW@8
_WriteProfileStringA@12
_WriteProfileStringA@12
_WriteProfileStringW@12
_WriteProfileStringW@12
_WriteTapemark@16
_WriteTapemark@16
_ZombifyActCtx@4
_ZombifyActCtx@4
__hread@12
__hread@12
__hwrite@12
__hwrite@12
__lclose@4
__lclose@4
__lcreat@8
__lcreat@8
__llseek@12
__llseek@12
__lopen@8
__lopen@8
__lread@12
__lread@12
__lwrite@12
__lwrite@12
_lstrcat@8
_lstrcat@8
_lstrcatA@8
_lstrcatA@8
_lstrcatW@8
_lstrcatW@8
_lstrcmp@8
_lstrcmp@8
_lstrcmpA@8
_lstrcmpA@8
_lstrcmpW@8
_lstrcmpW@8
_lstrcmpi@8
_lstrcmpi@8
_lstrcmpiA@8
_lstrcmpiA@8
_lstrcmpiW@8
_lstrcmpiW@8
_lstrcpy@8
_lstrcpy@8
_lstrcpyA@8
_lstrcpyA@8
_lstrcpyW@8
_lstrcpyW@8
_lstrcpyn@12
_lstrcpyn@12
_lstrcpynA@12
_lstrcpynA@12
_lstrcpynW@12
_lstrcpynW@12
_lstrlen@4
_lstrlen@4
_lstrlenA@4
_lstrlenA@4
_lstrlenW@4
_lstrlenW@4
